#!/usr/bin/env python3

## Reads in csvs containing image metadata for the individual training and test datasets
## Sets a cap on the number of images to include in the training/test sets to the minimum individual size
## Generates training and test datasets for each class for each combination of the 5 disturbance levels
##   by sampling from the combined individual datasets
## Creates files of images to be moved: df_train_[individ/pair etc]_[n].csv and df_test_[individ/pair etc]_[n].csv
## Then copies images from datapath directory to new directory 'combined_analysis' created in datapath directory

## Note: user needs to set file paths

### Created by Danielle Norman 30th November 2021

############### Imports ################

import pandas as pd
import numpy as np
import subprocess
from pandas.io.parsers import read_csv
import itertools
import shutil

##### File paths ######

filein = '/a/b/c/' # path to image metadata csvs
datapath = '/x/y/z/' # path to image directory

### Form 'combined' datasets by sampling from training data used in individual analysis
### Each set is formed to contain the same number of training images, with an even split between the disturbance levels included.
### Note: due to rouding in division, sets vary +/1 in total images

################### Create combined training sets ##########################

# Read in individual dataset dataframes
df_veg1 = read_csv(filein + 'individ_df_train_final1.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg2 = read_csv(filein + 'individ_df_train_final2.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg3 = read_csv(filein + 'individ_df_train_final3.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg4 = read_csv(filein + 'individ_df_train_final4.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg5 = read_csv(filein + 'individ_df_train_final5.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]

# Identify smallest training set from the individual disturbance levels as size of training sets for combined analysis
train_cap = min(len(df_veg1),len(df_veg2),len(df_veg3),len(df_veg4),len(df_veg5))
#train_cap

# Concat to form one dataframe
df_veg = pd.concat([df_veg1,df_veg2,df_veg3,df_veg4,df_veg5])
class_list = df_veg.class_group.unique()
#len(df_veg)

# Create list of combined levels
set_levels = ['individ','pair','triple','quad','all']

# Loop over combinations and sample from disturbance levels within dataframe
for i in range(1,6): # pair - triple - quad - all
    imgs_reqd = round(train_cap/i) # split # images required evenly
    df_set = pd.DataFrame() # initialise
    for v in range(1,6): # loop over disturbance levels and sample imgs_reqd for each
        num_spc = 0
        while num_spc < len(class_list): # sample until all species included
            df_tmp = df_veg[df_veg.vegetation_score==v].sample(imgs_reqd)
            num_spc = len(df_tmp.class_group.unique())
        df_set = pd.concat([df_set, df_tmp]) # build dataframe containing sampled sets for individual disturbance levels
    df_veg=df_set # update pooled data to sample from in next iteration
    len(df_veg)
    for s in itertools.combinations(range(1,6),i): # loop over possible combinations at i's level, i.e. all pairs or all triples, etc.
        df_s = pd.DataFrame(columns=df_set.columns)
        df_str = ''
        for elem in s: # combine required disturbance levels to form dataset
            df_s = pd.concat([df_s,df_set[df_set.vegetation_score==elem]])
            df_str = df_str + '_' + str(elem)
        len(df_s)
        df_s.to_csv(filein + 'combinations/df_train_' + set_levels[i-1] + df_str + '.csv') # save df


##################### Sample test sets to ensure consistent size #######################
# Read in individual dataset dataframes
df_veg1_test = read_csv(filein + 'individ_df_test_final1.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg2_test = read_csv(filein + 'individ_df_test_final2.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg3_test = read_csv(filein + 'individ_df_test_final3.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg4_test = read_csv(filein + 'individ_df_test_final4.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]
df_veg5_test = read_csv(filein + 'individ_df_test_final5.csv',index_col=0)[['SourceFile','camera_alias','vegetation_score','species','class_group']]

# Identify smallest training set from the individual disturbance levels as size of training sets for combined analysis
test_cap = min(len(df_veg1_test),len(df_veg2_test),len(df_veg3_test),len(df_veg4_test),len(df_veg5_test))
# test_cap

# Concat to form one dataframe
df_veg_test = pd.concat([df_veg1_test,df_veg2_test,df_veg3_test,df_veg4_test,df_veg5_test])
class_list = df_veg_test.class_group.unique()
#len(df_veg_test)

for v in range(1,6):
    df_test = df_veg_test[df_veg_test.vegetation_score==v]
    num_spc=0
    while num_spc < len(class_list): # sample until all species included
        df_samp = df_test.sample(test_cap)
        num_spc = len(df_samp.class_group.unique())
    df_samp.to_csv(filein + 'combinations/df_test_individ_' + str(v) + '.csv')


################## Copy files ###########################

# set_levels = ['individ','pair','triple','quad','all'] # reminder

# Generate list of all possible combinations
set_list = list()
for v in range(1,6):
    set_list.extend(list(itertools.combinations(range(1,6),v)))

# for i in range(1,6): # pair - triple - quad - all
for s in set_list:
    lvl = set_levels[len(s)-1]
    df_str=''
    for elem in s: # combine required disturbance levels to form dataset string
        df_str = df_str + '_' + str(elem)
    df_train = read_csv(filein + 'combinations/df_train_' + lvl + df_str + '.csv') # read in training df
    if lvl == 'individ': 
        df_test = read_csv(filein + 'combinations/df_test_' + lvl + df_str + '.csv') # read in test df
    # Move images
    for class_item in class_list:
        train_paths = list(df_train[df_train.class_group==class_item].SourceFile)
        print("Moving", s , " ", class_item, "images now")
        # Create destination directories
        directory_training = datapath + "combined_analysis/" + lvl + df_str + "/training/" + class_item
        subprocess.check_call("mkdir -p " + directory_training, shell=True)
        # Copy images
        for file in train_paths:
            filename = file.rsplit('/',1)[1]
            shutil.copyfile(file,directory_training+'/'+filename) ##########
        if lvl == 'individ':
            test_paths = list(df_test[df_test.class_group==class_item].SourceFile)
            directory_test = datapath + "combined_analysis/" + lvl + df_str + "/test/" + class_item
            subprocess.check_call("mkdir -p " + directory_test, shell=True)
            for file in test_paths:
                filename = file.rsplit('/',1)[1]
                shutil.copyfile(file,directory_test+'/'+filename) ##########