#!/usr/bin/env python3

## Reads in 'dataset_cumulative.csv' containing image metadata
## Restricts classes to those captured on at least 2 cameras in each disturbance level
## Generates training and test datasets for each species class for each disturbance level
##   using a 90 %:10 % camera split training:test
## Creates files of images to be moved: individ_[n]_train_list.csv and individ_[n]_test_list.csv
## Then copies images from datapath directory to new directory 'individ_analysis' created in datapath directory

## Note: user needs to set file paths

### Created by Danielle Norman 30th November 2021

############### Imports ################

import pandas as pd
import numpy as np
import subprocess
import random
from pandas.io.parsers import read_csv
import shutil

########## File paths - User to set ###########

filein = '/a/b/c/' # path to image metadata csv
datapath = '/d/e/f/' # path to image directory
outpath = '/x/y/z/' # path to directory in which to store extra csvs produced, i.e. no. cams per disturbance level

##### Function ######
# Function to sample per class to training max 4500 and test max 200 images per class

def split_dataset(train_list,test_list):
    """
    Returns training and test set pathways, with appropriate sizes.
    The training set pathways are returned first, then the test set pathways.
    Input a grouped pandas object.
    """
# Count number of images for species class
    num_train = len(train_list)
    num_test = len(test_list)
# Set cap per class
    train_max = 4500
    test_max = 500
    ## Extract training filepaths
    if(num_train > train_max):
        train_paths = list(train_list.sample(train_max).SourceFile)
    else:
        train_paths = list(train_list.SourceFile)
    ## Extract test filepaths 
    if(num_test > test_max):
        test_paths = list(test_list.sample(test_max).SourceFile)
    else:
        test_paths = list(test_list.SourceFile)
    train_count = len(train_paths)
    test_count = len(test_paths)
    return train_paths, test_paths, train_count, test_count

##### Read in csv containing all data #####           

df_sort = pd.read_csv(filein+'dataset_cumulative.csv',index_col=0)
df_sort.columns
# len(df_sort)
# Remove images from later sampling, i.e. in folder "camera_trap_deployments"
df_sort['ind_deploy'] = df_sort.SourceFile.str.find('camera_trap_deployments')
df_sort = df_sort[df_sort.ind_deploy <0]
# len(df_sort)

## Check all species in all disturbance levels are captured on at least 2 cameras
df_num_cams = df_sort.groupby(['vegetation_score','class_group','camera_alias']).size().groupby(level=[0,1]).size()
df_num_cams = df_num_cams[df_num_cams>=2]
# df_num_cams.to_csv('outpath+'num_cams_per_spc_per_veg.csv')

# Restrict to common species which have been captured on at least 2 cameras in each disturbance level
list_common = list(['beardedpig','human','landfowl','leopardcat','lessermousedeer','malayancivet','malayanporcupine','maskedpalmcivet','pangolin','pigtailedmacaque','orangutan','redmuntjac','sambardeer','sunbear'])
df_sort = df_sort[df_sort.class_group.isin(list_common)]

######## Split data into disturbance levels and sample cameras into training vs test ##################

## Select 10% of cameras to be test set for each disturbance level, ensuring all species feature and the set contains ~10% images (i.e. 0.09<N<0.11)

## Loop over disturbance levels creating df for train and test sets for each
# initialise df to store number of cameras and images
num_images = pd.DataFrame(index=range(1,6),columns=['train_cams','test_cams','train_images','test_images'])
for v in range(1,6):
    # Generate list of cameras per disturbance level
    df_veg = df_sort[df_sort.vegetation_score==v] # subset to form disturbance level dataset
    list_cam = list(df_veg.camera_alias.unique()) # extract camera list
    len(list_cam)

    # Create test set by randomly sampling cameras such that all species are included
    num_spc_test = 0 # initialise
    num_spc_train = 0 # initialise
    df_test = df_veg.iloc[0] # initialise for while loop condition
    counter=0
    while num_spc_test < len(list_common) or num_spc_train < len(list_common) or (len(df_test)/len(df_veg))<0.09 or (len(df_test)/len(df_veg))>0.11:
            test_list = random.sample(list_cam,round(0.1*len(list_cam))) # randomly select 10% of cameras
            df_test = df_veg[df_veg.camera_alias.isin(test_list)] # create test dataframe of images
            num_spc_test = len(df_test.class_group.unique()) # count # species in test set
            # print(num_spc_test)
            # len(df_test)/len(df_veg)
            df_train = df_veg[~df_veg.camera_alias.isin(test_list)] # form training set from remaining cameras
            num_spc_train = len(df_train.class_group.unique()) #  count # species in training set
            # print(num_spc_train)
            # len(df_train)/len(df_veg)
            for c in df_test.class_group.unique():
                if len(df_test[df_test.class_group==c]) > len(df_train[df_train.class_group==c]):
                    num_spc_test=0
                    break
            counter+=1
    print(num_spc_test)
    len(df_test)/len(df_veg)
    print(num_spc_train)
    len(df_train)/len(df_veg)
    # save dataframes
    df_train.to_csv(filein+'individ' + str(v) + '_train_list.csv')
    df_test.to_csv(filein+'individ' + str(v) + '_test_list.csv')
    num_images.iloc[v-1] = [len(df_train.camera_alias.unique()),len(test_list),len(df_train),len(df_test)]

    #print(counter)
    # check proportions
    len(df_test)/len(df_veg) + len(df_train)/len(df_veg) == 1
# num_images.to_csv(outpath+'individ_img_cam_count.csv')

# Check species distributions

###### Sample classes to form final training and test sets #######
# 
# Within training/test camera sets, sample for each class to a train max of 4500 and test max 500 images per class.
# Create destination directories and copy images

## Loop over disturbance levels to create datasets and copy files

for v in range(1,6):
# Load train and test dataframes
    df_train = read_csv(filein + 'individ' + str(v) + '_train_list.csv',index_col=0)
    df_test = read_csv(filein + 'individ' + str(v) + '_test_list.csv',index_col=0)
    # len(df_train.camera_alias.unique())
    # len(df_test.camera_alias.unique())
# Initialise dataframes to store final datasets
    df_train_final=pd.DataFrame(columns=df_train.columns)
    df_test_final=pd.DataFrame(columns=df_test.columns)
# Extract class list
    class_list = df_train["class_group"].unique().tolist()
    num_images_spc = pd.DataFrame(index=class_list,columns=['train','test'])
# Loop over classes
    for class_item in class_list:
    # Get individual species group
        train_class = df_train[df_train.class_group==class_item]
        test_class = df_test[df_test.class_group==class_item]
    # Get training and test set pathways
        train_paths, test_paths, train_count, test_count = split_dataset(train_class,test_class)
        num_images_spc.loc[class_item]=[train_count, test_count]
    # Add to dataset dfs for storage
        tmp_train = df_train[df_train.SourceFile.isin(train_paths)]
        tmp_test = df_test[df_test.SourceFile.isin(test_paths)]
        df_train_final = pd.concat([df_train_final,tmp_train])
        df_test_final = pd.concat([df_test_final,tmp_test])
    # Move images
        print("Moving", class_item, "images now")
        # Create destination directories
        directory_training = datapath + "individ_analysis/" + str(v) + "/training/" + class_item
        directory_test = datapath + "individ_analysis/" + str(v) + "/test/" + class_item
        subprocess.check_call("mkdir -p " + directory_training, shell=True)
        subprocess.check_call("mkdir -p " + directory_test, shell=True)
        # Copy images
        for file in train_paths:
            filename = file.rsplit('/',1)[1]
            shutil.copyfile(file,directory_training+'/'+filename) ##########
        for file in test_paths:
            filename = file.rsplit('/',1)[1]
            shutil.copyfile(file,directory_test+'/'+filename) ##########
#        len(df_train.camera_alias.unique())
#        len(df_test.camera_alias.unique())
    num_images_spc.to_csv(filein + 'num_images_individ' + str(v) + '.csv')
    df_train_final.to_csv(filein + 'individ_df_train_final' + str(v) + '.csv')
    df_test_final.to_csv(filein + 'individ_df_test_final' + str(v) + '.csv')