#!/usr/bin/env python3
## Example script for evaluating network performance on HPC
## Reads in model checkpoint file (hdf5 file) and images from test directory
## Also reads in csvs containing the number of files in training/test directories for each class (num_imges.csv)
##    and a csv containing test image metadata (image_list_with_cumulative.csv)
## Outputs predictions and metrics file for each network, as well as a summary file
## Uses Keras

## Note: user to set path

###### IMPORTS #######
import numpy as np
import os
import sys
import pandas as pd
from sklearn import metrics
from sklearn.metrics import classification_report
import h5py
import sys
from keras.preprocessing.image import ImageDataGenerator
from keras.models import load_model
from keras.optimizers import RMSprop, SGD, Adam
from keras.callbacks import LearningRateScheduler, ModelCheckpoint

############## SET PATHS AND SETTINGS ###################
networks = ['vgg16','resnet50','inceptionv3']
# chkpts = ['','_full']
chkpts = ['_full']
runs = ['', '_batch64', '_Adamcustom', '_RMSpropcustom', '_freeze', '_dropout25', '_dropout75']

path = sys.argv[1]

## Pair network and runs for dataframe index
ind = []
for network in networks:
    for run in runs:
        ind.append(network + run)

# labels = ['baseline','batch size: 64','optimiser: Adam + amended LR','optimiser: RMSprop + amended LR','frozen convolutional layers','dropout probability: 0.25','dropout probability: 0.75','weight decay']
## Load checkpoints and compile model
for chkpt in chkpts:
    df = pd.DataFrame(data=np.nan,index= ind,columns=['T1','T5','F1','T1_seq'])
    for network in networks:
        for run in runs:
            ## Identify hyperparameter settings
            if run.find('Adam')==1:
                opt = 'Adam'
            elif run.find('RMSprop')==1:
                opt = RMSprop()
            else:
                opt = SGD(momentum=0.9)
            if run == '_batch64':
                batch = 64
            else:
                batch = 128
            ## Import and compile model
            nnet = load_model(path + 'checkpoints'+chkpt+'/checkpoints_'+ network + run + '.hdf5')
            nnet.compile(
                optimizer = opt,
                loss='categorical_crossentropy',
                metrics=['accuracy', 'top_k_categorical_accuracy']
                )
            ################ Load test set #####################
            ## Read in test data
            test_datagen = ImageDataGenerator(rescale=1./255)
            test_set = test_datagen.flow_from_directory(
                path + 'test',
                target_size=(224, 224),
                batch_size=batch,
                class_mode='categorical',
                shuffle=False) # files sorted into alphanumeric order

            test_files = sum(len(os.listdir(path+'test/'+d)) for d in os.listdir(path+'test'))
            
            ## Read in test image metadata csv
            image_list = pd.read_csv(path + 'image_list_with_cumulative.csv',index_col=0) 
            image_list = image_list.sort_index()

            ## Extract filenames into dataframe and assign cumulative number from image_list file
            predictions=pd.DataFrame(test_set.filenames)
            predictions["order"] = predictions.index # store current order for matching guess to later
            predictions.index = predictions[0].str.rsplit('/',1,expand=True)[1] # extract filename to index
            predictions=predictions.sort_index() # sort to ensure in same order as in image_list
            predictions["cumulative"] = image_list.cumulative
            ## return to original order to ensure predictions stored accurately
            predictions = predictions.sort_values(by="order")
            
            ## Evaluate and run predictions
            scores = nnet.evaluate_generator(test_set,steps=np.ceil(test_files/batch))
            true_labels = test_set.classes
            test_set.reset()
            predict = nnet.predict_generator(test_set,steps=np.ceil(test_files/batch))
            predict_labels = predict.argmax(axis=-1)
            f1 = metrics.f1_score(true_labels, predict_labels, average='macro')
            # print("Full dataset:\n Loss: ", scores[0], "\n Top-1 Acc: ", scores[1], "\n Top-5 Acc: ", scores[2], "\n F-1 Score: ", f1)

            ## seq eval
            predictions['answer'] = true_labels
            predictions['guess1'] = predict_labels
            predictions["class"] = predictions[0].str.split('/',1,expand=True)[0]
            predictions.to_csv(path + 'eval_output/' + network + run + '_predictions.csv')

            ## Loop over classes and store whether sequence has been correctly evaluated i.e. one image correct
            class_list = predictions["class"].unique().tolist()
            df_event = pd.DataFrame(index = predictions["cumulative"].unique().tolist())
            df_event["correct"] = 0
            df_event["class"] = ""

            for class_item in class_list:
                class_group = predictions.groupby(['class']).get_group(class_item) # extract class group images 
                event_list = class_group["cumulative"].unique().tolist() # list of events and number of images per event
                for event in event_list:
                    event_group = class_group.groupby(['cumulative']).get_group(event)
                    df_event["class"].loc[event] = class_item
                    if sum(event_group.guess1==event_group.answer)>0:
                        df_event.correct.loc[event] = 1
        
            t1_seq = sum(df_event.correct)/len(df_event)
            df.loc[network + run] = [scores[1], scores[2], f1, t1_seq]

            ## Extract labels
            label_map = (test_set.class_indices) # dictionary containing species index and species label
            species_index = list(label_map.values())
            species_labels = list(label_map.keys())

            ## Use classification_report to generate precision, recall and f-1
            results = classification_report(true_labels, predict_labels, target_names=species_labels, digits=5,output_dict=True)
            results_species = dict(list(results.items())[0:len(species_index)]) # species results
            results_overall = dict(list(results.items())[len(species_index):]) # micro, macro, weighted average results

            ### Extract metrics
            ## Convert results per species dictionary to pandas dataframe
            df_run = pd.DataFrame(results_species).transpose()

            ## Count number of training images per species
            num_images = pd.read_csv(path + 'num_images.csv',index_col=0)
            num_images.sort_index(inplace=True)

            ## Add number of training images to dataframe in column 'train'
            df_run['key'] = species_index
            df_run['train'] = num_images['train']
            df_run.to_csv(path + 'eval_output/' + network + run + '_df.csv')
            
    df.to_csv(path + 'run_summary' + chkpt + '.csv')
 
