#!/usr/bin/env python3

### Example code for running Inceptionv3 through Keras
### Inputs are provided through shell script:
# sys.argv[1]: path to dataset training images
# sys.argv[2]: path to dataset validation images
# sys.argv[3]: path to directory to store interim checkpoint files
# sys.argv[4]: path to store epoch metrics csv
# sys.argv[5]: path to directory to store final checkpoint files 


####### Imports ##########
import timeit
import os
import numpy as np
import pandas as pd
import h5py # added
import sys # added
from keras.regularizers import l2
from keras.applications import InceptionV3 
from keras.preprocessing.image import ImageDataGenerator
from keras.models import Model, model_from_json
from keras.layers import Dense, Flatten, Dropout, GlobalAveragePooling2D
from keras.optimizers import SGD, Adam
from keras.callbacks import LearningRateScheduler, ModelCheckpoint

## Count number of classes for network final layer dimensions
num_classes = (len(next(os.walk(sys.argv[1]))[1]))

## Start timer
tic=timeit.default_timer()

## Load and augment training images
train_datagen = ImageDataGenerator(
        rescale=1./255,
        shear_range=0.2,
        horizontal_flip=True,
	brightness_range=[0.8,1.2])
# zoom_range=0.1,

training_set = train_datagen.flow_from_directory(sys.argv[1],
    target_size=(224, 224),
    batch_size=128,
    class_mode='categorical')
#    shuffle=False)

## Load validation images
test_datagen = ImageDataGenerator(rescale=1./255)

test_set = test_datagen.flow_from_directory(sys.argv[2],
    target_size=(224, 224),
    batch_size=128,
    class_mode='categorical')
#    shuffle=False)

## Load model pretrained on ImageNet
base_model = InceptionV3(weights='imagenet',
                   include_top=False,
                   input_shape=(224, 224, 3))

## Complete the model
x = base_model.output
x = GlobalAveragePooling2D()(x)
# x = Dense(4096, activation='relu', name='fc4096_1')(x)
# x = Dropout(0.5)(x)
# x = Dense(4096, activation='relu', name='fc4096_2')(x)
x = Dropout(0.5)(x)
custom_out = Dense(num_classes, activation='softmax', name='softmax')(x)


# Create the final architecture.
model = Model(inputs=base_model.input, outputs=custom_out)
print(model.summary())
model.save_weights("tmpi.h5")

## Set learning rate
def scheduler(epoch):
    if epoch <= 5:
        lr = 0.0001
        return lr
    elif epoch > 5 and epoch <= 10:
        lr = 0.0001
        return lr
    elif epoch > 10 and epoch <= 20:
        lr = 0.00005
        return lr
    elif epoch > 20 and epoch <= 30:
        lr = 0.00001
        return lr
    elif epoch > 30 and epoch <= 35:
        lr = 0.000005
        return lr
    elif epoch > 35 and epoch <= 40:
        lr = 0.000001
        return lr
    else:
        lr = 0.000001
        return lr

dynamic_lr = LearningRateScheduler(scheduler, verbose=2)

##  Save the model and weights whenever there is an improvement in accuracy
filepath = sys.argv[3]
checkpoint = ModelCheckpoint(filepath, monitor='val_acc', verbose=2, save_best_only=True, mode='max')

## Save the LearningRateScheduler and the Checkpoint as list for callback
callbacks_list = [checkpoint, dynamic_lr]

## Add weight decay
regularizer = l2(0.0005 / 2)
for layer in model.layers:
    for attr in ['kernel_regularizer', 'bias_regularizer']:
        if hasattr(layer, attr) and layer.trainable:
            setattr(layer, attr, regularizer)

out = model_from_json(model.to_json())
out.load_weights("tmpi.h5", by_name=True)
model=out

## Compile the model
model.compile(optimizer='Adam',
              loss='categorical_crossentropy',
              metrics=['accuracy'])

## Train the model
history = model.fit_generator(training_set,
                    steps_per_epoch=len(training_set),
                    validation_steps=len(test_set),
                    epochs=40,
                    validation_data=test_set,
                    callbacks=callbacks_list)

toc=timeit.default_timer()

hist_df = pd.DataFrame(history.history)
hist_df.to_csv(sys.argv[4])

filepath = sys.argv[5]
model.save(filepath)

print(toc-tic)
