
#!/usr/bin/env python3
# Author: Danielle Norman, based on script by Philipp Bischoff
# Description: reads in csv containing raw data images, and outputs a csv listing the images to be included in the baseline dataset
# Inputs: image_info.csv: image metadata including filename, tags, seq no., data/time
#         camera_info.csv: camera info including vegetation scores [provided by Dr Ollie Wearn]
# Output: dataset_final.csv; dataset_cumulative_csv

##########################

import pandas as pd
import numpy as np

##########################

####### Data ########

## Read csv files into panda dataframes
df_image = pd.read_csv('../Data/image_info.csv')
df_camera = pd.read_csv('../Data/camera_info.csv')

## Remove non-tagged images [assumed empty]
df_image['Subject'].replace('', np.nan, inplace=True)
df_image.dropna(subset=['Subject'], inplace=True)

## Remove unwanted images: multi-species, unknown, setup, malfunction and pickup
pattern_unwanted = r'Out\sOf\sView|Setup|1(2)|Malfunction|Time-lapse|Pick-up|Undetermined|Unknowable|Unknown'
df_image = df_image[df_image.Subject.str.contains(pattern_unwanted) == False]
filter = df_image['Subject'].str.contains(pattern_unwanted)
df_image = df_image[~filter]

## Extract camera position: alias found by following the pattern, i.e. 1+ letters, followed by numbers, dash, letters and/or number, dash, numbers 
df_image['camera_alias'] = df_image['Subject'].str.extract('(?<=15:\s)(.+?)(?=,)')

## Extract species
df_image['species'] = df_image['Subject'].str.extract('(?<=\s1:\s)(.+?)(?=$)') # find anything after 1:
df_image['species'] = df_image['species'].str.extract('(?<=^)(.+?)(?=,|$)') # find everything up to a comma or end of line (if end of line it is redundant)
df_image.loc[df_image['Subject'].str.contains('16:\sPerson|16:\sPeople|1:\sHunter', regex=True), 'species'] = "Human" # unify human labels 

## Label remaining images as 'empty'
df_image['species'].fillna('Empty', inplace=True)

## Remove spaces/punctuation and make lower case for consistent labelling
df_image['species'] = df_image['species'].str.lower().str.replace('[^a-zA-Z]', '')

## Remove unwanted species and images with non-species specific tags
species_unwanted = r'pillmillipede|^spinyrat|^ants|^bat|^skink|^eagle|^frog|^dragonfly|^owl|^lizard|^porcupinesp|^pitta|^civet|^birdsp|^bird|^cat|^ratsp|^mousedeersp|^muntjacsp|^pittasp|^treeshrewsp|^unidentifiedtreeshrew|^squirrel'
# df_image.species.str.contains(species_unwanted).sum()
df_image = df_image[df_image.species.str.contains(species_unwanted) == False]

## Unify tags
df_image.species[df_image.species == 'tembedau'] = 'banteng'
df_image.species[df_image.species == 'borneanorangutan'] = 'orangutan'
df_image.species[df_image.species == 'dog'] = 'domesticdog'
df_image.species[df_image.species == 'malaycivet'] = 'malayancivet'
df_image.species[df_image.species == 'malayporcupine'] = 'malayanporcupine'
df_image.species[df_image.species == 'magpierobin'] = 'orientalmagpierobin'
df_image.species[df_image.species == 'coucal'] = 'greatercoucal'
df_image.species[df_image.species == 'sundapangolin'] = 'pangolin'
df_image.species[df_image.species == 'monitorlizard'] = 'roughneckmonitor'

###### Vegetation #######

## Extract camera alias and veg score from camera_info df
df_camera = df_camera[['Alias (TrapID)', 'Vegetation Score']]
df_camera.columns = ['camera_alias', 'vegetation_score']

## Remove images without camera tag
df_image['camera_alias'].replace('', np.nan, inplace=True)
df_image.dropna(subset=['camera_alias'], inplace=True)

## Create dictionary and add veg score for each image
dict_camera = dict(df_camera.to_dict('split')['data'])
df_image['vegetation_score'] = df_image['camera_alias'].map(dict_camera)

## Note: some cameras do not have a veg index - they will be removed in subsequent script

###### Taxonomy #######

## Remove reptiles
species_unwanted = r'asiantortoise|^reticulatedpython|^roughneckmonitor'
df_image = df_image[df_image.species.str.contains(species_unwanted) == False]

## Create groups
def assign_species(df):
    if (df["species"] == "orangutan" or df["species"] == "borneangibbon" or df["species"] == "westerntarsier"):
            return("ape")
    elif (df["species"] == "bandedlinsang"):
            return("malayancivet")
    elif (df["species"] == "bandedcivet" or df["species"] == "hosescivet"):
            return("bandedpalmcivet")
    elif (df["species"] == "earspotsquirrel" or df["species"] == "plantainsquirrel" or df["species"] == "prevostssquirrel"):
            return("beautifulsquirrels")
    elif (df["species"] == "pentailtreeshrew" or df["species"] == "longfootedtreeshrew"):
            return("commontreeshrew")
    elif (df["species"] == "redleggedcrake" or df["species"] == "whitebreastedwaterhen" or df["species"] == "greategret"):
            return("crane")
    elif (df["species"] == "greatercoucal" or df["species"] == "borneangroundcuckoo" or df["species"] == "crestedserpenteagle"):
            return("cuckoo")
    elif (df["species"] == "emeralddove" or df["species"] == "spotteddove"):
            return("dove")
    elif (df["species"] == "crestedfireback" or df["species"] == "greatargus" or df["species"] == "bulwerspheasant" or df["species"] == "chestnutnecklacedhillpartridge" or df["species"] == "bluebreastedquail" or df["species"] == "redjunglefowl" or df["species"] == "roulroul"):
            return("landfowl")
    elif (df["species"] == "maroonlangur" or df["species"] == "hoseslangur"):
            return("langur")
    elif (df["species"] == "slendersquirrel"):
            return("lowssquirrel")
    elif (df["species"] == "mullersrat" or df["species"] == "smallspinyrat" or df["species"] == "blackrat" or df["species"] == "whiteheadsrat"):
            return("muellersgiantsundarat")
    elif (df["species"] == "blackcappedbabbler" or df["species"] == "littlespiderhunter" or df["species"] == "whitecrownedshama" or df["species"] == "shorttailedbabbler" or df["species"] == "whitecrownedforktail" or df["species"] == "orientalmagpierobin" or df["species"] == "yellowventedbulbul" or df["species"] == "chestnutmunia" or df["species"] == "moustachedbabbler" or df["species"] == "borneangroundbabbler" or df["species"] == "giantpitta" or df["species"] == "blueheadedpitta" or df["species"] == "bandedpitta" or df["species"] == "blackandcrimsonpitta" or df["species"] == "hoodedpitta"):
            return("songbird")
    elif (df["species"] == "fourstripedgroundsquirrel" or df["species"] == "leastpygmysquirrel" or df["species"] == "tuftedgroundsquirrel" or df["species"] == "flyingsquirrel"):
            return("stripedgroundsquirrel")
    elif (df["species"] == "orientalsmallclawedotter" or df["species"] == "malayweasel"):
            return("yellowthroatedmarten")

df_image["class_group"] = df_image.apply(lambda x: assign_species(x), axis=1)
df_image.loc[df_image['class_group'].isnull(),'class_group'] = df_image['species']

## Save to csv
df_image.to_csv('../Data/dataset_final.csv')

###### Create dataset with unique event numbers for keeping sequences intact #########

## Remove images without sequence info
df_image['Sequence'].replace('', np.nan, inplace=True)
df_image.dropna(subset=['Sequence'], inplace=True)

## Sort images by camera_alias and EventNumber to ensure sequential images are together
df_sort = df_image.sort_values(by=['camera_alias','EventNumber'])
df_sort["cumulative"]=""

## Loop over dataframe and assign Cumulative EventNumbers, i.e. event per whole dataset rather than per camera
cum = 1 # initialise cumulative event counter
count=1 # initialise image counter
df_sort["cumulative"].iloc[0]=cum # assign first image cumulative event 1
ev = int(df_sort["EventNumber"].iloc[0]) # initialise event number from first row
cam = df_sort["camera_alias"].iloc[0] # initialise camera_alias from first row
for index,row in df_sort.iloc[1:,:].iterrows():
    if row['EventNumber'] != '' and row['camera_alias'] != '':
        if row["EventNumber"] != ev or row["camera_alias"] != cam: # if new event
            cum+=1
            if row["EventNumber"] != ev:
                ev = row["EventNumber"] # update EventNumber tracker
            if row["camera_alias"] != cam:
                cam = row["camera_alias"] # update camera_alias tracker
        df_sort["cumulative"].iloc[count]=cum # assign cumulative event
    count+=1

## Save updated dataframe to csv    
df_sort.to_csv('../Data/dataset_cumulative.csv')
